/* $Id: defs.h,v 1.4 1991/07/11 21:14:11 jtsillas Exp $ */

/*****************************************************************************
 *
 *  xdbx - X Window System interface to the dbx debugger
 *
 *  Copyright 1989 The University of Texas at Austin
 *  Copyright 1990 Microelectronics and Computer Technology Corporation
 *
 *  Permission to use, copy, modify, and distribute this software and its
 *  documentation for any purpose and without fee is hereby granted,
 *  provided that the above copyright notice appear in all copies and that
 *  both that copyright notice and this permission notice appear in
 *  supporting documentation, and that the name of The University of Texas
 *  and Microelectronics and Computer Technology Corporation (MCC) not be 
 *  used in advertising or publicity pertaining to distribution of
 *  the software without specific, written prior permission.  The
 *  University of Texas and MCC makes no representations about the 
 *  suitability of this software for any purpose.  It is provided "as is" 
 *  without express or implied warranty.
 *
 *  THE UNIVERSITY OF TEXAS AND MCC DISCLAIMS ALL WARRANTIES WITH REGARD TO
 *  THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS, IN NO EVENT SHALL THE UNIVERSITY OF TEXAS OR MCC BE LIABLE FOR
 *  ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER
 *  RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF
 *  CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
 *  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 *  Author:  	Po Cheung
 *  Created:   	March 10, 1989
 * 
 *****************************************************************************
 * 
 *  xxgdb - X Window System interface to the gdb debugger
 *  
 * 	Copyright 1990 Thomson Consumer Electronics, Inc.
 *  
 *  Permission to use, copy, modify, and distribute this software and its
 *  documentation for any purpose and without fee is hereby granted,
 *  provided that the above copyright notice appear in all copies and that
 *  both that copyright notice and this permission notice appear in
 *  supporting documentation, and that the name of Thomson Consumer
 *  Electronics (TCE) not be used in advertising or publicity pertaining
 *  to distribution of the software without specific, written prior
 *  permission.  TCE makes no representations about the suitability of
 *  this software for any purpose.  It is provided "as is" without express
 *  or implied warranty.
 *
 *  TCE DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
 *  ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT
 *  SHALL TCE BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES
 *  OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 *  WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 *  ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 *  SOFTWARE.
 *
 *  Adaptation to GDB:  Pierre Willard
 *  XXGDB Created:   	December, 1990
 *
 *****************************************************************************/

/*  defs.h
 *
 *    Contain #includes, #defines and typedefs
 */

#include <stdio.h>
#include <sys/param.h>
#include <X11/Xos.h>
#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>
#include <X11/Xatom.h>
#include <X11/Shell.h>
#include <X11/cursorfont.h>
#include <Xm/Xm.h>

#ifndef AssignMax
#define AssignMax(x, y) 	if ((y) > (x)) x = (y)
#endif
#ifndef AssignMin
#define AssignMin(x, y) 	if ((y) < (x)) x = (y)
#endif

#define NHELPLABELS	50
#define NHELPBUTTONS    50
#define LINESIZ         512	/* input line length */
#define MAXNAME         256	/* max identifier length */
#define MAXARGS 	20	/* max number of args */
#define ADD_SIZE 	16	/* # of files added during Realloc */
#define CHARS_PER_LINE  20	/* estimated number of chars per line */
#define ADD_LINES       50	/* # of lines to be added in realloc */
#define NTOKENS		6	/* number of tokens */
#define DEBUGGER      	"gdb"	/* name of executable */
#define XDBXPROMPT  	"(mxgdb) " /* xdbx prompt string */
#define DELIMITERS	" !%^&*()+=~|;:{},/#<?\"\n\t"

#define BREAK_HELP	"Please select a function name or an address, or put cursor on a line to break at"
#define CLEAR_HELP	"Please select a stop sign to clear"
#define PRINT_HELP	"Please select an expression to print"
#define UNDISPLAY_HELP	"Please select a number to undisplay"
#define DISPLAY_HELP    "Please select ax expression to display"
#define SEARCH_HELP	"Nothing to search"

#define	LASTCH(s)	(s[strlen(s)-1])
#define	SECLASTCH(s)	(s[strlen(s)-2])

#define _SCROLLDOWN ((int *)1)	/* These are used as callback arguments */
#define _SCROLLUP ((int *)2)	/* for the vertical scroll back and for the */
#define _SCROLLUPPAGE ((int *)3) /* left/right page buttons. */
#define _SCROLLDOWNPAGE ((int *)4)
#define _SCROLLDRAG ((int *)5)
#define _SCROLLRIGHT ((int *) 6)
#define _SCROLLLEFT ((int *) 7)
#define _SCROLLDROP ((int *) 8)
#define _PAGELEFT ((int *) 9)
#define _PAGERIGHT ((int *) 10)

typedef struct {		/* Structure which defines a button */
  Widget Button;		/* within the pulldown menu heirarchy  */
  char *Label;			/* in the menu bar. This is used by */
  char *Argument;		/* the Command heirarchy and the Help */
  char *RN;			/* heirarchy. Button specifies the  */
  void (*CB)();			/* widget, Label is text placed in the */
  char *accel;			/* button, Argument is passed to the */
  char *acceltext;		/* callback (CB XmNactivateCallback), */
} MenuButtonStruct;		/* RN is the resource name, accel and */
				/* acceltext are the accelerator KeyPress */
				/* and text, respectively. */

typedef struct {		/* Structure which defines a pulldown menu */
  Widget CB;			/* pane within the oulldown menu heirarchy. */
  Widget PM;			/* CB is the CascadeButton widget, PM is the */
  char *Label;			/* PM is the PulldownMenu, Label is the */
  char *RNPM;			/* button's label, RNPM and RNCB are the */
  char *RNCB;			/* resource names for the PM and CB, Buttons */
  MenuButtonStruct *Buttons;	/* points to a MenuButton Struct for the */
  int Nbuttons;			/* child buttons, Nbuttons is the number of */
  char mnemonic;		/* button children, mnemonic is the mnemonic */
} PMCBStruct;			/* KeySym. */
  

typedef struct {
    Boolean   bell;		/* if True, bell on */
    Boolean   displayWindow;	/* if True, display window on startup */
    String    delimiters;	/* set of delimiters for word selection */
    String    prompt;		/* prompt string for xdbx */

    Pixel     stop_color;	/* color of stop sign */
    Pixel     arrow_color;	/* color of arrow sign */
    Pixel     updown_color;	/* color of updown sign */
    Pixel     bomb_color;	/* color of bomb sign */

    Dimension dataDpyMaxHeight;	/* data display window maximum height */
    Dimension dataDpyMaxWidth;	/* data display window maximum width */

    Boolean   bigicon;		/* xdbx option -bigicon */
    Boolean   debug;		/* xdbx option -debug */

    Boolean   dbxopt_r;		/* dbx option -r */
    Boolean   dbxopt_i;		/* dbx option -i */
    String    includeDir;	/* dbx option -I includeDir */
    Boolean   dbxopt_k;		/* dbx option -k */
    String    cfile;		/* Berkeley  dbx option -c file */
    Boolean   dbxopt_kbd;	/* Sun dbx option -kbd */
    String    fcount;		/* SunOS 4.0 dbx option -f fcount */
    String    startup;		/* SunOS 4.0 dbx option -s startup */
    String    tstartup;		/* SunOS 4.0 dbx option -sr tstartup */
    Boolean   pixie;		/* Mips dbx option -pixie */
} XdbxResources;

typedef struct {
    char 		*filename;	/* name of file */
    char 		*pathname;	/* full path name of file */
    char 		*buf;		/* buffer holding source file */
    long		filesize;	/* size of file in bytes */
    time_t		mtime;		/* time last modified */
    int	 		lines;		/* # of lines on display */
    int	 		currentline;	/* line where caret is */
    int	 		topline;	/* top line number in the window */
    int	 		bottomline;	/* bottom line number in window */
    int	 		lastline;	/* number of lines in source file */
    XmTextPosition 	topPosition;	/* top display position of buffer */
    XmTextPosition 	*linepos;	/* array of text pos for each newline*/
} FileRec, *FileRecPtr;

typedef struct {
    char	*mesg;			/* part of matched string */
    unsigned	stop;			/* stop number */
    char 	*func;			/* function name */
    int		line;			/* line number */
    char	*file;			/* file name */
    char	*display;		/* variable display output */
} Tokens;

typedef struct {
    char			*pat;		/* regular expression */
    struct re_pattern_buffer	*buf;		/* buffer for compile regex */
    int				reg_token[NTOKENS];	/* register number */
} PatternRec, *PatternRecPtr;

typedef struct commandRec {
    char		*command;
    struct commandRec	*next;
} CommandRec, *CommandRecPtr;

typedef struct {
    Cardinal	i;			/* index to arrowsign[] */
    char	file[MAXNAME];		/* file associated with */
    int	 	line;			/* line number */
    char 	func[MAXNAME];		/* function name associated with */
} Arrow;

typedef struct {
    Cardinal	i;			/* index to updownsign[] */
    char	file[MAXNAME];		/* file associated with */
    int	 	line;			/* line number */
    char 	func[MAXNAME];		/* function name associated with */
} Updown;

typedef struct {
    char	*file;			/* file associated with */
    int	 	line;			/* line number of stop */
    unsigned    tag;                    /* used in deleting stops */
} Stops;

typedef struct {
    Cardinal	i;			/* index to bombsign[] */
    char	file[MAXNAME];		/* file associated with */
    int	 	line;			/* line number */
    char 	func[MAXNAME];		/* function name associated with */
} Bomb;


typedef struct {
    Widget      w;
    Boolean     mapped;
} ArrowSign;

typedef struct {
    Widget      w;
    Boolean     mapped;
} UpdownSign;

typedef struct {
    Widget      w;
    Boolean     mapped;
} StopSign;

typedef struct {
    Widget      w;
    Boolean     mapped;
} BombSign;
